`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: NUTS AND VOLTS MAGAZINE
// Engineer: FRED EADY
// 
// Create Date:    10:47:01 07/25/2008 
// Design Name: 
// Module Name:    lcd_drvr 
// Project Name: FPGA PRIMER
// Target Devices: XILINX XC3S50A WITH 10.0 MHZ CLOCK
// Tool versions: 
// Description: DRIVES DIGILENT PMODCLP LCD AND BLINKS DAS LEDS
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module lcd_drvr(
    input clk_10mhz,
	 input reset,
	 output reg lcd_e,
	 output reg lcd_rs,
	 output reg [7:0] lcd_data,
    output reg [7:0] led_out
    );

reg	[3:0]		lcd_state;		//current LCD driver state
reg	[19:0]	lcd_clk;			//LCD function timer
reg	[8:0]		char_ptr;		//points to next character to display on LCD
reg	[8:0]		char2display;	//the character to display on the LCD
reg 	[22:0] 	clk_divider;	//used to slow down LED blink rate

parameter	cmd_function_set = 8'h3C,
				cmd_display_set = 8'h0C,
				cmd_display_clr	= 8'h01,
				cmd_entry_mode	= 8'h06,
				cmd_display_home = 8'h02,
				cmd_display_line2 = 8'hC0;
				
parameter	power_up = 4'h0,
				function_set = 4'h1,
				display_set = 4'h2,
				display_clr = 4'h3,
				display_home = 4'h4,
				display_msg = 4'h5,
				msg_table = 4'h6,
				display_line2 = 4'h7,
				idle = 4'h8;

always @(posedge clk_10mhz)
begin
	clk_divider <= clk_divider + 1;
	if(clk_divider[22])
	 begin
		clk_divider[22] <= 0;
		led_out <= led_out + 1;
	 end
end 	
				
always @(posedge clk_10mhz or negedge reset)
	begin
		if(reset == 0)
		begin
			lcd_e <= 0;
			lcd_rs <= 0;
			lcd_clk <= 0;
			lcd_state <= function_set;
			char_ptr <= 1;
		end
		else
			begin
		case(lcd_state)
		
		power_up:
		if (lcd_clk == 20'h4E210)
			begin
				lcd_clk <= 0;
				lcd_state <= function_set;
			end
		else
			begin
				lcd_data <= cmd_function_set;
				lcd_rs <= 0;
				lcd_e <= 0;
				lcd_state <= power_up;
				lcd_clk <= lcd_clk + 1;
			end
	
		function_set:
		begin
			if(lcd_clk == 400) //40 uS
				begin
					lcd_clk <= 0;
					lcd_state <= display_set;
				end
			else if(lcd_clk == 20)
				begin
					lcd_e <= 0;
					lcd_state <= function_set;
					lcd_clk <= lcd_clk + 1;
				end
			else if(lcd_clk == 10)
				begin
					lcd_e <= 1;
					lcd_state <= function_set;
					lcd_clk <= lcd_clk + 1;
				end
			else
				begin
					lcd_rs <= 0;
					lcd_data <= cmd_function_set;
					lcd_state <= function_set;
					lcd_clk <= lcd_clk + 1;
				end
		end
							
		display_set:
		begin
			if(lcd_clk == 400) //40 uS
				begin
					lcd_clk <= 0;
					lcd_state <= display_clr;
				end
			else if(lcd_clk == 20)
				begin
					lcd_e <= 0;
					lcd_state <= display_set;
					lcd_clk <= lcd_clk + 1;
				end
			else if(lcd_clk == 10)
				begin
					lcd_e <= 1;
					lcd_state <= display_set;
					lcd_clk <= lcd_clk + 1;
				end
			else
				begin
					lcd_rs <= 0;
					lcd_data <= cmd_display_set;
					lcd_state <= display_set;
					lcd_clk <= lcd_clk + 1;
				end
		end
		
		display_clr:
		begin
			if(lcd_clk == 16400) //1.64 mS
				begin
					lcd_clk <= 0;
					lcd_state <= display_home;
				end
			else if(lcd_clk == 20)
				begin
					lcd_e <= 0;
					lcd_state <= display_clr;
					lcd_clk <= lcd_clk + 1;
				end
			else if(lcd_clk == 10)
				begin
					lcd_e <= 1;
					lcd_state <= display_clr;
					lcd_clk <= lcd_clk + 1;
				end
			else
				begin
					lcd_rs <= 0;
					lcd_data <= cmd_display_clr;
					lcd_state <= display_clr;
					lcd_clk <= lcd_clk + 1;
				end
		end
	
		display_home:
		begin
			if(lcd_clk == 16400) //1.64 mS
				begin
					lcd_clk <= 0;
					lcd_state <= msg_table;
				end
			else if(lcd_clk == 20)
				begin
					lcd_e <= 0;
					lcd_state <= display_home;
					lcd_clk <= lcd_clk + 1;
				end
			else if(lcd_clk == 10)
				begin
					lcd_e <= 1;
					lcd_state <= display_home;
					lcd_clk <= lcd_clk + 1;
				end
			else
				begin
					lcd_rs <= 0;
					lcd_data <= cmd_display_home;
					lcd_state <= display_home;
					lcd_clk <= lcd_clk + 1;
			end
		end
			
		display_line2:
		begin
			if(lcd_clk == 16400) //1.64 mS
				begin
					lcd_clk <= 0;
					lcd_state <= msg_table;
					char_ptr <= char_ptr + 1;
				end
			else if(lcd_clk == 20)
				begin
					lcd_e <= 0;
					lcd_state <= display_line2;
					lcd_clk <= lcd_clk + 1;
				end
			else if(lcd_clk == 10)
				begin
					lcd_e <= 1;
					lcd_state <= display_line2;
					lcd_clk <= lcd_clk + 1;
				end
			else
				begin
					lcd_rs <= 0;
					lcd_data <= cmd_display_line2;
					lcd_state <= display_line2;
					lcd_clk <= lcd_clk + 1;
				end
		end
	
		display_msg:
		begin
			if(lcd_clk == 400) //40 uS
				begin
					lcd_clk <= 0;
					lcd_state <= msg_table;
					char_ptr <= char_ptr + 1;
				end
			else if(lcd_clk == 20)
				begin
					lcd_e <= 0;
					lcd_state <= display_msg;
					lcd_clk <= lcd_clk + 1;
				end
			else if(lcd_clk == 10)
				begin
					lcd_e <= 1;
					lcd_state <= display_msg;
					lcd_clk <= lcd_clk + 1;
				end
			else
				begin
					lcd_rs <= 1;
					lcd_data <= char2display;
					lcd_state <= display_msg;
					lcd_clk <= lcd_clk + 1;
				end
		end
		
		msg_table:
			begin
				if(char_ptr == 17)
					begin
						lcd_clk <= 0;
						lcd_state <= display_line2;
					end
				else if(char_ptr == 34)
					begin
						lcd_clk <= 0;
						lcd_state <= idle;
					end
				else
					begin
						case(char_ptr)
						//LINE 1
						1:char2display <= " "; 	 
						2:char2display <= "N"; 	
						3:char2display <= "U";	
						4:char2display <= "T";	
						5:char2display <= "S";	
						6:char2display <= " ";	
						7:char2display <= "A";	
						8:char2display <= "N";	
						9:char2display <= "D";	
						10:char2display <= " ";		
						11:char2display <= "V";	
						12:char2display <= "O";	
						13:char2display <= "L";	
						14:char2display <= "T";	
						15:char2display <= "S"; 
						16:char2display <= " "; 
						//LINE 2
						18:char2display <= " "; 
						19:char2display <= " ";	
						20:char2display <= "F";	
						21:char2display <= "P";	
						22:char2display <= "G";	
						23:char2display <= "A";	
						24:char2display <= "S";	
						25:char2display <= " ";		
						26:char2display <= "R";	
						27:char2display <= "O";	
						28:char2display <= "C";	
						29:char2display <= "K";	
						30:char2display <= "!"; 
						31:char2display <= "!";
						32:char2display <= " ";
						33:char2display <= " ";
						endcase 
						
						lcd_clk <= 0;
						lcd_state <= display_msg;
					end
			end
			
		idle:
			begin
				lcd_clk <= 0;
			end

	endcase
	 end
	end
endmodule
